@@ -1,6 +1,7 @@
 /***********************************************************
 (C) Copyright 2003 A.M. Kuchling.  All Rights Reserved
 (C) Copyright 2004 A.M. Kuchling, Ralph Heinkel  All Rights Reserved
+(C) Copyright 2013-2017 Sandro Mani  All Rights Reserved
 
 Permission to use, copy, modify, and distribute this software and its
 documentation for any purpose and without fee is hereby granted,
@@ -10,7 +11,7 @@ supporting documentation, and that the name of A.M. Kuchling and
 Ralph Heinkel not be used in advertising or publicity pertaining to
 distribution of the software without specific, written prior permission.
 
-A.M. KUCHLING, R.H. HEINKEL DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
+A.M. KUCHLING, R.H. HEINKEL S. MANI DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
 SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS,
 IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY SPECIAL, INDIRECT OR
 CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF
@@ -52,6 +53,8 @@ typedef struct {
 
 static PyTypeObject SaneDev_Type;
 
+static int g_sane_initialized = 0;
+
 
 /* Raise a SANE exception */
 static PyObject *
@@ -66,7 +69,7 @@ PySane_Error(SANE_Status st)
 static void
 SaneDev_dealloc(SaneDevObject *self)
 {
-  if(self->h)
+  if(self->h && g_sane_initialized)
     sane_close(self->h);
   self->h = NULL;
   PyObject_DEL(self);
@@ -658,7 +661,7 @@ PySane_init(PyObject *self, PyObject *args)
   SANE_Status st = sane_init(&version, NULL);
   if(st != SANE_STATUS_GOOD)
     return PySane_Error(st);
-
+  g_sane_initialized = 1;
   return Py_BuildValue("iiii", version,
                        SANE_VERSION_MAJOR(version),
                        SANE_VERSION_MINOR(version),
@@ -672,6 +675,7 @@ PySane_exit(PyObject *self, PyObject *args)
     return NULL;
 
   sane_exit();
+  g_sane_initialized = 0;
   Py_INCREF(Py_None);
   return Py_None;
 }
